"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.onAnalyseProject = void 0;
var fs = require("fs-extra");
var ts = require("typescript");
var node_1 = require("../../../node");
function compile(fileNames) {
    return fileNames.map(function (fileName, index) {
        var fileContent = fs.readFileSync(fileName).toString();
        var result = ts.transpileModule(fileContent, { compilerOptions: { module: ts.ModuleKind.CommonJS } });
        return "\n      var mock" + index + " = (()=>{\n        var exports = {}\n        " + result.outputText + "\n        return exports.default\n      })();\n";
    });
}
function onAnalyseProject(mockFilesPath) {
    var mocks = compile(mockFilesPath);
    node_1.pri.serviceWorker.pipe(function (text) {
        return "\n      " + text + "\n      " + mocks.join('\n') + "\n\n      var allMocks = {" + mockFilesPath
            .map(function (__, index) {
            return "...mock" + index;
        })
            .join(',') + "}\n      var mockList = []\n\n      Object.keys(allMocks).forEach(mockKey => {\n        var mock = allMocks[mockKey]\n        mockList.push({ url: new URL(mockKey, location), value: mock })\n      })\n\n      self.addEventListener(\"fetch\", event => {\n        var requestUrl = new URL(event.request.url)\n\n        const mockInfo = mockList.find(mock => requestUrl.hostname === mock.url.hostname &&\n           requestUrl.pathname === mock.url.pathname)\n\n        if (mockInfo) {\n          var responseInit = { status: 200, statusText: \"OK\", \n            headers: { \"Content-Type\": \"application/json\", \"Access-Control-Allow-Origin\": \"*\" } }\n\n          if (typeof mockInfo.value === 'function') {\n            event.respondWith(\n              Promise.resolve(mockInfo.value()).then(responseBody => {\n                return new Response(JSON.stringify(responseBody), responseInit);\n              })\n            );\n          } else {\n            var responseBody = mockInfo.value;\n            var mockResponse = new Response(JSON.stringify(responseBody), responseInit);\n            event.respondWith(mockResponse);\n          }\n        }\n      })\n    ";
    });
}
exports.onAnalyseProject = onAnalyseProject;
//# sourceMappingURL=compile-mock.js.map