"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.babelPluginTransformImport = void 0;
/* eslint-disable no-param-reassign */
function getReplacements(state) {
    if (state.opts instanceof Array) {
        return state.opts;
    }
    if (state.opts && state.opts.replacements instanceof Array) {
        return state.opts.replacements;
    }
    return [state.opts];
}
function babelPluginTransformImport(_a) {
    var t = _a.types;
    var source = function (value, state, pipeImport) { return t.stringLiteral(pipeImport(value, state.filename)); };
    return {
        visitor: {
            ImportDeclaration: function (path, state) {
                var replacements = getReplacements(state);
                replacements.forEach(function (_a) {
                    var pipeImport = _a.pipeImport, removeCssImport = _a.removeCssImport;
                    var value = path.node.source.value;
                    if (removeCssImport) {
                        var cssRegex = '^(.+?)\\.(css|sass|scss|less)$';
                        var cssPattern = new RegExp("^(" + cssRegex + "|" + cssRegex + "/.*)$");
                        if (cssPattern.test(value)) {
                            path.remove();
                        }
                    }
                    else {
                        path.node.source = source(value, state, pipeImport);
                    }
                });
            },
            ExportDeclaration: function (path, state) {
                var replacements = getReplacements(state);
                replacements.forEach(function (_a) {
                    var pipeImport = _a.pipeImport;
                    if (path.node.source) {
                        var value = path.node.source.value;
                        path.node.source = source(value, state, pipeImport);
                    }
                });
            },
            CallExpression: function (path, state) {
                var replacements = getReplacements(state);
                replacements.forEach(function (_a) {
                    var pipeImport = _a.pipeImport;
                    var node = path.node;
                    if (node.callee.name === 'require' &&
                        node.arguments &&
                        node.arguments.length === 1 &&
                        t.isStringLiteral(node.arguments[0])) {
                        path.node.arguments = [source(node.arguments[0].value, state, pipeImport)];
                    }
                });
            },
        },
    };
}
exports.babelPluginTransformImport = babelPluginTransformImport;
//# sourceMappingURL=babel-plugin-transfer-import.js.map