"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.transferCommandsArrayToMap = void 0;
var _ = require("lodash");
var log_1 = require("./log");
// Transfer commands array to commands map
function transferCommandsArrayToMap(commandRegisters) {
    var duplicates = getDuplicatesCommand(commandRegisters);
    if (duplicates.length > 0) {
        log_1.logFatal("Duplicate commands:\n" + duplicates
            .map(function (duplicate) {
            return duplicate;
        })
            .join('\n'));
    }
    return createRootCommandRegisters(commandRegisters);
}
exports.transferCommandsArrayToMap = transferCommandsArrayToMap;
function createRootCommandRegisters(commandRegistersSplitName) {
    var rootNames = _.uniq(commandRegistersSplitName.map(function (commandRegisterSplitName) {
        return commandRegisterSplitName.name[0];
    }));
    var rootCommandRegisters = commandRegistersSplitName.filter(function (commandRegisterSplitName) {
        return commandRegisterSplitName.name.length === 1;
    });
    var leafCommandRegisters = commandRegistersSplitName.filter(function (commandRegisterSplitName) {
        return commandRegisterSplitName.name.length > 1;
    });
    return rootNames.map(function (rootName) {
        var childs = leafCommandRegisters
            .filter(function (leafCommandRegister) {
            return leafCommandRegister.name[0] === rootName;
        })
            .map(function (leafCommandRegister) {
            return __assign(__assign({}, leafCommandRegister), { name: leafCommandRegister.name.slice(1, leafCommandRegister.name.length) });
        });
        var commandRegisterExactByName = rootCommandRegisters.find(function (parentCommandRegister) {
            return parentCommandRegister.name[0] === rootName;
        });
        if (commandRegisterExactByName) {
            return __assign(__assign({}, commandRegisterExactByName), { childs: createRootCommandRegisters(childs) });
        }
        return {
            name: [rootName],
            childs: createRootCommandRegisters(childs),
        };
    });
}
function getDuplicatesCommand(commandRegisters) {
    return _.filter(commandRegisters.map(function (commandRegister) {
        return commandRegister.name;
    }), function (value, index, iteratee) {
        return _.includes(iteratee, value, index + 1);
    });
}
//# sourceMappingURL=commands.js.map