"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ProjectConfig = exports.GlobalState = exports.ProjectInfo = void 0;
var ProjectInfo = /** @class */ (function () {
    function ProjectInfo() {
        this.routes = [];
        this.hasConfigFile = false;
        this.hasLayout = false;
        this.has404File = false;
        this.stores = [];
    }
    return ProjectInfo;
}());
exports.ProjectInfo = ProjectInfo;
/**
 * Types for globalState
 */
var GlobalState = /** @class */ (function () {
    function GlobalState() {
        /**
         * Selected source type.
         * 'root' or some packages.
         */
        this.selectedSourceType = 'root';
        this.projectPackageJson = {};
        /**
         * Development enviroment.
         */
        this.isDevelopment = false;
        /**
         * packages info
         */
        this.packages = [];
        /**
         * plugins info
         */
        this.plugins = [];
    }
    return GlobalState;
}());
exports.GlobalState = GlobalState;
/**
 * Types for priconfig.json
 */
var ProjectConfig = /** @class */ (function () {
    function ProjectConfig() {
        /**
         * Title for html <title>.
         */
        this.title = null;
        /**
         * custom host
         */
        this.host = 'localhost';
        /**
         * Dev server port, when execute npm start.
         */
        this.devPort = null;
        /**
         * Output main file name.
         * Only take effect on `type=project`
         */
        this.outFileName = 'index.js';
        /**
         * Output main css file name.
         * Only take effect on `type=project`
         */
        this.outCssFileName = 'index.css';
        /**
         * Css extract.
         */
        this.cssExtract = false;
        /**
         * Bundle file name
         */
        this.bundleFileName = 'bundle.js';
        /**
         * Bundle library target
         */
        this.bundleLibraryTarget = 'umd';
        /**
         * Specify the development url, work both for `npm start` and `npm run preview`.
         * In most scenes, it should not be configured.
         * > Conflict with `devPort`
         */
        this.devUrl = null;
        /**
         * Dist dir path.
         * Only take effect on `npm run build` | `pri build`.
         */
        this.distDir = 'dist';
        /**
         * Assets public path. `"https://www.some.com"`, `"https://www.some.com/somePath"`, `"/somePath"`.
         * If not set, result: `/<distPath>`.
         * If set /somePath for example, result: `/somePath/<distPath>`.
         * If set some.com for example, result: `https://www.some.com/<distPath>`.
         * If set some.com/somePath for example, result: `https://www.some.com/somePath/<distPath>`.
         * Only take effect on `npm run build` | `pri build`.
         */
        this.publicPath = '/';
        /**
         * Base href for all pages.
         * For example, `/admin` is the root path after deploy, you should set baseHref to `/admin`.
         * There is no need to modify the code, routing `/` can automatically maps to `/admin`.
         * Only take effect on `npm run build` | `pri build`
         */
        this.baseHref = '/';
        /**
         * Using https for server.
         */
        this.useHttps = true;
        /**
         * Use service worker
         * Warning: if disable it, mocks, prefetch, serverRender will become invalid.
         */
        this.useServiceWorker = false;
        /**
         * Client server render
         * Warning: depend on service worker, should set useServiceWorker=true first.
         */
        this.clientServerRender = false;
        /**
         * Custom routes. When this configuration exists, it will not parse the `pages` directory.
         */
        this.routes = [];
        /**
         * Enable hash router.
         */
        this.useHashRouter = false;
        /**
         * Suggestion to open!
         */
        this.unexpectedFileCheck = true;
        /**
         * Enable package lock.
         */
        this.packageLock = false;
        /**
         * Hide source code when publish npm package.
         * Only take effect on `projectType = component`.
         */
        this.hideSourceCodeForNpm = false;
        /**
         * Watch node_modules
         */
        this.watchNodeModules = false;
        /**
         * Check deps * ^ ~
         */
        this.allowDepsSemver = null;
        /**
         * Enable incremental lint check.
         */
        this.incrementalLint = false;
        this.npmClient = 'npm';
        /**
         * Enable hot loader
         */
        this.hotReload = true;
        /**
         * Project Root Dom Id
         */
        this.projectRootId = 'root';
        /**
         * Use default html template
         */
        this.useHtmlTemplate = true;
        /**
         * Enable ensure files
         */
        this.ensureFiles = true;
        /**
         * is material component
         */
        this.materialComponent = false;
        /*
         * extra vendors defined in priConfig, which you dont want to rebundle during project rebuid
         */
        this.extraVendors = [];
        /**
         * Unnecessarily published packages, only take effect when includeAll is true.
         */
        this.unPublishList = [];
    }
    return ProjectConfig;
}());
exports.ProjectConfig = ProjectConfig;
//# sourceMappingURL=define.js.map