"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateCertificate = void 0;
var fs = require("fs-extra");
var forge = require("node-forge");
var path = require("path");
var global_state_1 = require("./global-state");
var structor_config_1 = require("./structor-config");
function generateCertificate() {
    var cacheDir = path.join(path.join(global_state_1.globalState.projectRootPath, structor_config_1.tempPath.dir) + "/ssl");
    var privateKeyPath = path.join(cacheDir, 'private.pem');
    var certPath = path.join(cacheDir, 'primary.crt');
    var cachedKey = fs.existsSync(privateKeyPath) && fs.readFileSync(privateKeyPath);
    var cachedCert = fs.existsSync(certPath) && fs.readFileSync(certPath);
    if (cachedKey && cachedCert) {
        return {
            key: cachedKey,
            cert: cachedCert,
        };
    }
    var pki = forge.pki;
    var keys = pki.rsa.generateKeyPair(2048);
    var cert = pki.createCertificate();
    cert.publicKey = keys.publicKey;
    cert.serialNumber = '02';
    cert.validity.notBefore = new Date();
    cert.validity.notAfter = new Date();
    cert.validity.notAfter.setFullYear(cert.validity.notBefore.getFullYear() + 1);
    var attrs = [
        {
            name: 'commonName',
            value: 'prijs.org',
        },
        {
            name: 'countryName',
            value: 'China',
        },
        {
            shortName: 'ST',
            value: 'Virginiae',
        },
        {
            name: 'localityName',
            value: 'Blacks',
        },
        {
            name: 'organizationName',
            value: 'pri',
        },
        {
            shortName: 'OU',
            value: 'pri test',
        },
    ];
    cert.setSubject(attrs);
    cert.setIssuer(attrs);
    cert.setExtensions([
        {
            name: 'basicConstraints',
            cA: true,
        },
        {
            name: 'keyUsage',
            keyCertSign: true,
            digitalSignature: true,
            nonRepudiation: true,
            keyEncipherment: true,
            dataEncipherment: true,
        },
        {
            name: 'extKeyUsage',
            serverAuth: true,
            clientAuth: true,
            codeSigning: true,
            emailProtection: true,
            timeStamping: true,
        },
        {
            name: 'nsCertType',
            client: true,
            server: true,
            email: true,
            objsign: true,
            sslCA: true,
            emailCA: true,
            objCA: true,
        },
        {
            name: 'subjectAltName',
            altNames: [
                {
                    type: 6,
                    value: 'http://pri.org/webid#me',
                },
                {
                    type: 7,
                    ip: '127.0.0.1',
                },
            ],
        },
        {
            name: 'subjectKeyIdentifier',
        },
    ]);
    cert.sign(keys.privateKey, forge.md.sha256.create());
    var privPem = pki.privateKeyToPem(keys.privateKey);
    var certPem = pki.certificateToPem(cert);
    fs.ensureDirSync(cacheDir);
    fs.writeFileSync(privateKeyPath, privPem);
    fs.writeFileSync(certPath, certPem);
    return {
        key: privPem,
        cert: certPem,
    };
}
exports.generateCertificate = generateCertificate;
//# sourceMappingURL=generate-certificate.js.map