"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __spreadArray = (this && this.__spreadArray) || function (to, from) {
    for (var i = 0, il = from.length, j = to.length; i < il; i++, j++)
        to[j] = from[i];
    return to;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.walkProjectFiles = void 0;
var path = require("path");
var walk = require("walk");
var global_state_1 = require("./global-state");
var structor_config_1 = require("./structor-config");
function walkProjectFiles() {
    return new Promise(function (resolve) {
        var gitIgnores = structor_config_1.gitIgnores.map(function (dir) {
            return path.join(global_state_1.globalState.projectRootPath, dir);
        });
        var scanIgnores = structor_config_1.ignoreScanFiles.map(function (addon) {
            return path.join(global_state_1.globalState.projectRootPath, addon);
        });
        var walker = walk.walk(global_state_1.globalState.projectRootPath, { filters: __spreadArray(__spreadArray([], gitIgnores), scanIgnores) });
        var files = [];
        walker.on('directories', function (root, dirStatsArray, next) {
            dirStatsArray.forEach(function (dirStats) {
                var dirPath = path.join(root, dirStats.name);
                if (dirPath === global_state_1.globalState.projectRootPath) {
                    // Skip project's root.
                    next();
                    return;
                }
                if (gitIgnores.concat(scanIgnores).some(function (ignorePath) {
                    return ignorePath === path.join(root, dirStats.name);
                })) {
                    next();
                    return;
                }
                files.push(__assign({ isDir: true }, path.parse(dirPath)));
                next();
            });
        });
        walker.on('file', function (root, fileStats, next) {
            if (gitIgnores.concat(scanIgnores).some(function (ignorePath) {
                return ignorePath === path.join(root, fileStats.name);
            })) {
                next();
                return;
            }
            files.push(__assign({ isDir: false }, path.parse(path.join(root, fileStats.name))));
            next();
        });
        walker.on('errors', function (root, nodeStatsArray, next) {
            next();
        });
        walker.on('end', function () {
            resolve(files);
        });
    });
}
exports.walkProjectFiles = walkProjectFiles;
//# sourceMappingURL=walk-project-files.js.map